package modeler;

import java.io.*;
import java.security.InvalidParameterException;
import java.util.Hashtable;
import java.util.Vector;

public class CNFFormula extends Formula {

	public class Clause implements Printable {

		public void setComment(String comment) {
			this.comment = comment;
		}

		public void addLiteral(Literal l) {
			if (l.getNo() == 0) {
				throw new InvalidParameterException("l");
			}

			this.literals.add(l);
			CNFFormula.this.addLiteral(l);
		}

		public void printOut(Writer out) throws IOException {
			/* if (this.comment != null) {
				out.write("c " + this.comment + "\n");
			}*/

			for (Literal l : this.literals) {
				l.printOut(out);
				out.write(' ');
			}
		}

		protected final Vector<Literal> literals = new Vector<Literal>();

		protected String comment = null;

	}

	public void printOut(Writer writer) throws IOException {
		writer.write(String.format("p cnf %1$s %2$s\n", this.getVariablesCount(), this.getClausesCount()));

		for (Clause c : this.clauses) {
			c.printOut(writer);
			writer.write("0\n");
		}

		writer.write("\n");
	}

	public int getClausesCount() {
		return this.clauses.size();
	}

	public int getLiteralsCount() {
		return this.literalsCount;
	}

	public int getVariablesCount() {
		return this.variables.size();
	}

	public void addClause(Clause clause) {
		this.clauses.add(clause);
	}

	protected final Vector<Clause> clauses = new Vector<Clause>();

	protected final Hashtable<Integer, VariableInfo> variables = new Hashtable<Integer, VariableInfo>();

	protected int literalsCount = 0;

	protected void addLiteral(Literal l) {
		int varNo = l.getVarNo();
		if (!this.variables.containsKey(varNo)) {
			this.variables.put(varNo, new VariableInfo(l));
			++this.literalsCount;

			assert this.variables.size() == varNo : "lvlala";
		} else {
			VariableInfo i = this.variables.get(varNo);
			if (!i.hasSign(l.isPositive())) {
				++this.literalsCount;
				i.setLiteral(l);
				assert this.literalsCount <= this.getVariablesCount() * 2 : "Invalid literals count.";
			}
		}
	}

	protected static class VariableInfo {

		public VariableInfo(Literal l) {
			if (l.isPositive()) {
				this.positive = l;
			} else {
				this.negative = l;
			}
		}

		public Literal getPositiveLiteral() {
			return this.positive;
		}

		public Literal getNegativeLiteral() {
			return this.negative;
		}

		public boolean hasPositive() {
			return this.positive != null;
		}

		public boolean hasNegative() {
			return this.negative != null;
		}

		public void setPositive(Literal l) {
			if (!l.isPositive()) {
				throw new InvalidParameterException("Invalid positive literal for variable.");
			}

			this.positive = l;
		}

		public void setNegative(Literal l) {
			if (l.isPositive()) {
				throw new InvalidParameterException("Invalid negative literal for variable.");
			}

			this.negative = l;
		}

		public boolean hasSign(boolean sign) {
			return sign ? this.hasPositive() : this.hasNegative();
		}

		public void setLiteral(Literal l) {
			if (l.isPositive()) {
				this.setPositive(l);
			} else {
				this.setNegative(l);
			}
		}

		protected Literal positive, negative;

	}

}
